// $Id: nextpleaseOptions.js,v 1.20 2010/02/07 Alexey Romanov $
nextplease.StringToKeyCodeMap = {
    "VK_LEFT": KeyEvent.DOM_VK_LEFT,
    "VK_RIGHT": KeyEvent.DOM_VK_RIGHT,
    "VK_UP": KeyEvent.DOM_VK_UP,
    "VK_DOWN": KeyEvent.DOM_VK_DOWN,
    "VK_PAGE_UP": KeyEvent.DOM_VK_PAGE_UP,
    "VK_PAGE_DOWN": KeyEvent.DOM_VK_PAGE_DOWN,
    "VK_HOME": KeyEvent.DOM_VK_HOME,
    "VK_END": KeyEvent.DOM_VK_END,
    "VK_F1": KeyEvent.DOM_VK_F1,
    "VK_F2": KeyEvent.DOM_VK_F2,
    "VK_F3": KeyEvent.DOM_VK_F3,
    "VK_F4": KeyEvent.DOM_VK_F4,
    "VK_F5": KeyEvent.DOM_VK_F5,
    "VK_F6": KeyEvent.DOM_VK_F6,
    "VK_F7": KeyEvent.DOM_VK_F7,
    "VK_F8": KeyEvent.DOM_VK_F8,
    "VK_F9": KeyEvent.DOM_VK_F9,
    "VK_F10": KeyEvent.DOM_VK_F10,
    "VK_F11": KeyEvent.DOM_VK_F11,
    "VK_F12": KeyEvent.DOM_VK_F12
};

nextplease.isAlpha = function (str) {
    var isAlphaRegEx = /^[a-z ]$/i;
    return isAlphaRegEx.test(str);
};

// Read the phrases that specify a next link
// by reading the preferences or defaults,
// populate the listbox with the phrases,
// and put the phrases in a lookup table.
nextplease.initListBox = function (listboxId, prefName, defaultValues) {
    var listbox = document.getElementById(listboxId);
    var phrase;
    var i = 0;
    var tempprefname = prefName + '.expr0';
    if (this.prefs.prefHasUserValue(tempprefname)) {
        while (this.prefs.prefHasUserValue(tempprefname)) {
            var prefValue = this.GetUnicharPref(tempprefname);
            var values = prefValue.split("|");
            for (var j = 0; j < values.length; j++) {
                phrase = values[j].replace(/&pipe;/g, "|");
                if (phrase !== "") {
                    listbox.appendItem(phrase, phrase);
    
                    // Scroll to the newly added item to workaround 
                    // what I think is a Firefox bug. I was getting
                    // Javascript exceptions when trying to read the
                    // values at the bottom that are "hidden".
                    listbox.ensureIndexIsVisible(listbox.getRowCount() - 1);
                }
            }
            i++;
            tempprefname = prefName + '.expr' + i;
        }
    } else {
        for (var k = 0; k < defaultValues.length; k++) {
            phrase = defaultValues[k];
            listbox.appendItem(phrase, phrase);
            // Scroll to item to workaround Firefox bug.
            listbox.ensureIndexIsVisible(k);
        }
    }
    listbox.ensureIndexIsVisible(0);
};

nextplease.initModifierSelector = function (prefname, elemId, defaultValue) {
    // Read the key binding prefs and set the dialog items appropriately
    var menuElem = document.getElementById(elemId);
    if (this.prefs.prefHasUserValue(prefname)) {
        menuElem.value = this.prefs.getCharPref(prefname);
    } else {
        // Special case check for setting prev modifier for upgraders
        // for version 0.5.x and earlier
        if ((elemId === 'nextpleasePrevModifier') && this.prefs.prefHasUserValue('nextplease.keymodifier')) {
            menuElem.value = this.prefs.getCharPref('nextplease.keymodifier');
        } else {
            menuElem.value = defaultValue;
        }
    }
};

nextplease.initKeySelector = function (prefname, keyElemId, keyMenuId, defaultValue) {
    var keyElem = document.getElementById(keyElemId);
    var keyMenu = document.getElementById(keyMenuId);
    if (this.prefs.prefHasUserValue(prefname)) {
        // test whether this is a special key
        var keyCode = this.prefs.getIntPref(prefname);
        if (this.KeyCodeToNameMap[keyCode]) {
            keyElem.value = "";
            keyMenu.value = this.KeyCodeToNameMap[keyCode];
        } else {
            keyMenu.value = "";
            keyElem.value = String.fromCharCode(keyCode);
        }
    } else {
        keyMenu.value = defaultValue;
        keyElem.value = "";
    }
};

nextplease.initRegEx = function (prefname, elemId, defaultValue) {
    var textbox = document.getElementById(elemId);
    if (this.prefs.prefHasUserValue(prefname)) {
        var prefValue = this.Trim(this.GetUnicharPref(prefname));
        if (prefValue !== "") {
            textbox.value = prefValue;
        } else {
            textbox.value = defaultValue;
        }
    } else {
        textbox.value = defaultValue;
    }
};

//Initialize options
nextplease.initializeOptions = function () {
    this.log("initializating options.");
    
    if (!this.prefs) {
        this.log("nextplease.prefs undefined. This shouldn't happen!");
        this.prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("");
    }

    this.keyBindingsChanged = false;
    this.initListBox('nextfields', 'nextplease.nextphrase', this.DefaultNextPhrases);
    this.initListBox('prevfields', 'nextplease.prevphrase', this.DefaultPrevPhrases);
    this.initListBox('firstfields', 'nextplease.firstphrase', this.DefaultFirstPhrases);
    this.initListBox('lastfields', 'nextplease.lastphrase', this.DefaultLastPhrases);

    this.initListBox('nextimagefields', 'nextplease.nextimage', this.DefaultNextImages);
    this.initListBox('previmagefields', 'nextplease.previmage', this.DefaultPrevImages);
    this.initListBox('firstimagefields', 'nextplease.firstimage', this.DefaultFirstImages);
    this.initListBox('lastimagefields', 'nextplease.lastimage', this.DefaultLastImages);

    this.logDetail("listboxes initialized.");

    document.getElementById('nextpleaseAllowSubmit').checked = this.prefs.getBoolPref('nextplease.allowsubmit');
    document.getElementById('nextpleaseAllowNumberShortcuts').checked = this.prefs.getBoolPref('nextplease.allownumbershortcuts');
    document.getElementById('nextpleaseAllowContextMenu').checked = this.prefs.getBoolPref('nextplease.allowcontextmenu');
    document.getElementById('nextpleaseAllowSmartNext').checked = this.prefs.getBoolPref('nextplease.allowsmartnext');

    this.logDetail("checkboxes initialized.");

    this.initModifierSelector('nextplease.keymodifier', 'nextpleaseModifier', 'CTRL+SHIFT');
    this.initModifierSelector('nextplease.prevkeymodifier', 'nextpleasePrevModifier', 'CTRL+SHIFT');
    this.initModifierSelector('nextplease.firstkeymodifier', 'nextpleaseFirstModifier', 'CTRL+SHIFT');
    this.initModifierSelector('nextplease.lastkeymodifier', 'nextpleaseLastModifier', 'CTRL+SHIFT');

    this.initKeySelector('nextplease.nextkey', 'nextkey', 'nextkeymenu', 'Right Arrow');
    this.initKeySelector('nextplease.prevkey', 'prevkey', 'prevkeymenu', 'Left Arrow');
    this.initKeySelector('nextplease.firstkey', 'firstkey', 'firstkeymenu', 'Up Arrow');
    this.initKeySelector('nextplease.lastkey', 'lastkey', 'lastkeymenu', 'Down Arrow');

    this.logDetail("keys initialized.");

    this.initRegEx('nextplease.nextregex', 'nextRegEx', this.DefaultNextRegEx);
    this.initRegEx('nextplease.prevregex', 'prevRegEx', this.DefaultPrevRegEx);
    this.initRegEx('nextplease.firstregex', 'firstRegEx', this.DefaultFirstRegEx);
    this.initRegEx('nextplease.lastregex', 'lastRegEx', this.DefaultLastRegEx);
    this.initRegEx('nextplease.galleryregex', 'galleryRegEx', this.DefaultGalleryRegEx);
    
    this.logDetail("regexes initialized.");

    document.getElementById("nextplease.log.normal").checked = this.prefs.getBoolPref('nextplease.log');
    document.getElementById("nextplease.log.detailed").checked = this.prefs.getBoolPref('nextplease.log.detailed');

    this.logDetail("options initialized.");
};

nextplease.saveKeyPref = function (prefname, keyElemId, keyMenuId) {
    var keyString = this.Trim(document.getElementById(keyElemId).value);
    if (keyString === "") {
        this.prefs.setIntPref(prefname, this.StringToKeyCodeMap[document.getElementById(keyMenuId).value]);
    } else {
        // if (!this.isAlpha(keyString)) return;
        var curCharCode = keyString.toUpperCase().charCodeAt(0);
        this.prefs.setIntPref(prefname, curCharCode);
    }
};

// Save the listbox items specified by listboxId into 
// the preference specified by prefs and prefbranchname
nextplease.saveListboxPrefs = function (listboxId, prefbranchname) {
    var listbox = document.getElementById(listboxId);
    var numRows = listbox.getRowCount();
    this.prefs.deleteBranch(prefbranchname + '.');
    var tempstr = "";
    for (var i = 0; i < numRows; i++) {
        tempstr = tempstr + "|" + listbox.getItemAtIndex(i).value.replace(/\|/g, "&pipe;");
    }
    this.SetUnicharPref(prefbranchname + '.expr0', tempstr);
};

// Write out the preferences from the dialog.
nextplease.saveOptions = function () {
    this.log("saving options.");
    
    this.prefs.setCharPref('nextplease.keymodifier', document.getElementById('nextpleaseModifier').value);
    this.prefs.setCharPref('nextplease.prevkeymodifier', document.getElementById('nextpleasePrevModifier').value);
    this.prefs.setCharPref('nextplease.firstkeymodifier', document.getElementById('nextpleaseFirstModifier').value);
    this.prefs.setCharPref('nextplease.lastkeymodifier', document.getElementById('nextpleaseLastModifier').value);

    this.saveKeyPref('nextplease.nextkey', 'nextkey', 'nextkeymenu');
    this.saveKeyPref('nextplease.prevkey', 'prevkey', 'prevkeymenu');
    this.saveKeyPref('nextplease.lastkey', 'lastkey', 'lastkeymenu');
    this.saveKeyPref('nextplease.firstkey', 'firstkey', 'firstkeymenu');

    this.logDetail("keys saved.");

    this.saveListboxPrefs('nextfields', 'nextplease.nextphrase');
    this.saveListboxPrefs('prevfields', 'nextplease.prevphrase');
    this.saveListboxPrefs('firstfields', 'nextplease.firstphrase');
    this.saveListboxPrefs('lastfields', 'nextplease.lastphrase');

    this.saveListboxPrefs('nextimagefields', 'nextplease.nextimage');
    this.saveListboxPrefs('previmagefields', 'nextplease.previmage');
    this.saveListboxPrefs('firstimagefields', 'nextplease.firstimage');
    this.saveListboxPrefs('lastimagefields', 'nextplease.lastimage');

    this.logDetail("phrases and images saved.");

    this.prefs.setBoolPref('nextplease.allowsubmit', document.getElementById("nextpleaseAllowSubmit").checked);
    this.prefs.setBoolPref('nextplease.allownumbershortcuts', document.getElementById("nextpleaseAllowNumberShortcuts").checked);
    this.prefs.setBoolPref('nextplease.allowcontextmenu', document.getElementById("nextpleaseAllowContextMenu").checked);
    this.prefs.setBoolPref('nextplease.allowsmartnext', document.getElementById("nextpleaseAllowSmartNext").checked);
    this.prefs.setBoolPref('nextplease.log', document.getElementById("nextplease.log.normal").checked);
    this.prefs.setBoolPref('nextplease.log.detailed', document.getElementById("nextplease.log.detailed").checked);

    this.logDetail("checkboxes saved.");

    this.SetUnicharPref('nextplease.nextregex', document.getElementById('nextRegEx').value);
    this.SetUnicharPref('nextplease.prevregex', document.getElementById('prevRegEx').value);
    this.SetUnicharPref('nextplease.firstregex', document.getElementById('firstRegEx').value);
    this.SetUnicharPref('nextplease.lastregex', document.getElementById('lastRegEx').value);
    this.SetUnicharPref('nextplease.galleryregex', document.getElementById('galleryRegEx').value);

    this.logDetail("regexes saved.");

    if (this.keyBindingsChanged) {
        alert("Changes to key binding will only take effect in new windows or after restarting Firefox.");
    }
    else {
        alert("To make changes take effect, run 'Next, Please' once either through the context menu or the old shortcut.");
    }

    this.logDetail("options saved.");
};

nextplease.onModifierMenuChange = function (keyElemId, menuKeyId, keyModifierId) {
    var keyElem = document.getElementById(keyElemId);
    var keyMenu = document.getElementById(menuKeyId);
    var keyModifierElem = document.getElementById(keyModifierId);

    var nextElem = document.getElementById('nextkey');
    var prevElem = document.getElementById('prevkey');
    var firstElem = document.getElementById('firstkey');
    var lastElem = document.getElementById('lastkey');

    var nextMenuElem = document.getElementById('nextkeymenu');
    var prevMenuElem = document.getElementById('prevkeymenu');
    var firstMenuElem = document.getElementById('firstkeymenu');
    var lastMenuElem = document.getElementById('lastkeymenu');

    var nextModifierElem = document.getElementById('nextpleaseModifier');
    var prevModifierElem = document.getElementById('nextpleasePrevModifier');
    var firstModifierElem = document.getElementById('nextpleaseFirstModifier');
    var lastModifierElem = document.getElementById('nextpleaseLastModifier');

    if ((keyModifierElem !== nextModifierElem) && (keyModifierElem.value === nextModifierElem.value) &&
            (keyElem.value.toUpperCase() === nextElem.value.toUpperCase()) && (keyMenu.value === nextMenuElem.value)) {
        keyElem.value = "";
        keyMenu.value = "";
    }

    if ((keyModifierElem !== prevModifierElem) && (keyModifierElem.value === prevModifierElem.value) &&
            (keyElem.value.toUpperCase() === prevElem.value.toUpperCase()) && (keyMenu.value === prevMenuElem.value)) {
        keyElem.value = "";
        keyMenu.value = "";
    }

    if ((keyModifierElem !== firstModifierElem) && (keyModifierElem.value === firstModifierElem.value) &&
            (keyElem.value.toUpperCase() === firstElem.value.toUpperCase()) && (keyMenu.value === firstMenuElem.value)) {
        keyElem.value = "";
        keyMenu.value = "";
    }

    if ((keyModifierElem !== lastModifierElem) && (keyModifierElem.value === lastModifierElem.value) &&
            (keyElem.value.toUpperCase() === lastElem.value.toUpperCase()) && (keyMenu.value === lastMenuElem.value)) {
        keyElem.value = "";
        keyMenu.value = "";
    }

    this.keyBindingsChanged = true;
    return true;
};

nextplease.onKeyMenuChange = function (keyElemId, menuKeyId, keyModifierId) {
    // Ensure that key and keymenu do not both
    // have an entry
    var keyElem = document.getElementById(keyElemId);
    var keyMenu = document.getElementById(menuKeyId);
    var keyModifierElem = document.getElementById(keyModifierId);
    if (keyMenu.value !== "") {
        keyElem.value = "";
    }

    var nextMenuElem = document.getElementById('nextkeymenu');
    var prevMenuElem = document.getElementById('prevkeymenu');
    var firstMenuElem = document.getElementById('firstkeymenu');
    var lastMenuElem = document.getElementById('lastkeymenu');

    var nextModifierElem = document.getElementById('nextpleaseModifier');
    var prevModifierElem = document.getElementById('nextpleasePrevModifier');
    var firstModifierElem = document.getElementById('nextpleaseFirstModifier');
    var lastModifierElem = document.getElementById('nextpleaseLastModifier');

    if ((keyMenu !== nextMenuElem) && (keyMenu.value === nextMenuElem.value) && (keyModifierElem.value === nextModifierElem.value)) {
        nextMenuElem.value = "";
    }
    if ((keyMenu !== prevMenuElem) && (keyMenu.value === prevMenuElem.value) && (keyModifierElem.value === prevModifierElem.value)) {
        prevMenuElem.value = "";
    }
    if ((keyMenu !== firstMenuElem) && (keyMenu.value === firstMenuElem.value) && (keyModifierElem.value === firstModifierElem.value)) {
        firstMenuElem.value = "";
    }
    if ((keyMenu !== lastMenuElem) && (keyMenu.value === lastMenuElem.value) && (keyModifierElem.value === lastModifierElem.value)) {
        lastMenuElem.value = "";
    }

    this.keyBindingsChanged = true;
    return true;
};

nextplease.onKeyChange = function (keyElemId, menuKeyId, keyModifierId) {
    var keyElem = document.getElementById(keyElemId);
    var keyModifierElem = document.getElementById(keyModifierId);
    if (!this.isAlpha(keyElem.value)) {
        keyElem.value = "";
        alert('You can only use A-Z for this field.');
        return false;
    }

    // Ensure that key and keymenu do not both
    // have an entry
    if (keyElem.value !== "") {
        document.getElementById(menuKeyId).value = "";
    }

    var nextElem = document.getElementById('nextkey');
    var prevElem = document.getElementById('prevkey');
    var firstElem = document.getElementById('firstkey');
    var lastElem = document.getElementById('lastkey');

    var nextModifierElem = document.getElementById('nextpleaseModifier');
    var prevModifierElem = document.getElementById('nextpleasePrevModifier');
    var firstModifierElem = document.getElementById('nextpleaseFirstModifier');
    var lastModifierElem = document.getElementById('nextpleaseLastModifier');

    if ((keyElem !== nextElem) && (nextElem.value.toUpperCase() === keyElem.value.toUpperCase()) && (keyModifierElem.value === nextModifierElem.value)) {
        nextElem.value = "";
    }
    if ((keyElem !== prevElem) && (prevElem.value.toUpperCase() === keyElem.value.toUpperCase()) && (keyModifierElem.value === prevModifierElem.value)) {
        prevElem.value = "";
    }
    if ((keyElem !== firstElem) && (firstElem.value.toUpperCase() === keyElem.value.toUpperCase()) && (keyModifierElem.value === firstModifierElem.value)) {
        firstElem.value = "";
    }
    if ((keyElem !== lastElem) && (lastElem.value.toUpperCase() === keyElem.value.toUpperCase()) && (keyModifierElem.value === lastModifierElem.value)) {
        lastElem.value = "";
    }

    this.keyBindingsChanged = true;
    return true;
};

// Add the an item from the field to the listbox specified by listboxId
nextplease.addToListbox = function (newElemFieldId, listboxId, map) {
    var newElemString = this.Trim(document.getElementById(newElemFieldId).value);

    if (newElemString === "") {
        alert("Trying to add whitespace string.");
    } else if (map[newElemString]) {
        alert("'" + newElemString + "' is already in the list.");
    } else {
        var listbox = document.getElementById(listboxId);
        listbox.appendItem(newElemString, newElemString);
        listbox.ensureIndexIsVisible(listbox.getRowCount() - 1);
        map[newElemString] = true;
    }
};

// Remove the selected item from the listbox specified by listboxId
nextplease.removeSelectedFromListbox = function (listboxId, map) {
    var listbox = document.getElementById(listboxId);

    var selected = listbox.selectedIndex;
    if (selected >= 0) {
        delete map[listbox.selectedItem.value];
        listbox.removeItemAt(selected);
    }
};

nextplease.restoreDefault = function (elementId, defaultString) {
    document.getElementById(elementId).value = defaultString;
};